/* Simple HTAccel library
 * Written Oct 2011 by Alex Clarke and Nova Scheidt for CS207 at the University of Regina
 *
 * Provides simple access to X-axis of HiTechnic acclerometers.
 * TODO: Add Y-axis support
 * BUGS: Z-axis values are not read correctly.
 *       Their range is incorrect, which can be easily fixed, but the data is also
 *       noisy.
 */

#include "HTAccel.h"
#include <inttypes.h>

const int HTAC = 1; //HT Accelerometer's I2C bus address is 1


/* readHTAccel_X()
 *
 * Returns X reading from Hi-Technic Accelerometer.
 * If sensor does not respond, last known X value is returned.
 * X values are constrained to [-200,200] representing -2G to 2G
 */
 
int readHTAccel_X()
{
  static int x = 0;   // Remember last x just in case read fails
 
  // Configure the HT Accelerometer
  Wire.beginTransmission(HTAC);// Prepare transmission for HTAC sensor
  
  Wire.write(0x42);            // send HTAC a single byte indicating what to 
                               // send in response to requestFrom command.
                               // 0x42 is the register on HTAC corresponding 
                               // to x_up. Other registers are in consecutive
                               // bytes.
                               
  uint8_t result = Wire.endTransmission();      //Actually send transmission
  if (result == 0) //Transmission succeeded
  {
    //Request information from Accelerometer
    Wire.requestFrom(HTAC, 6);  //Request 6 bytes from HT Accelerometer
    
    //Wait for information to arrive, or time out if it taks too long.
    int i = 0;
    while(Wire.available() < 6 && i < 10000)
    {
      i++;
    }
    
    //Read and interpret received information if no time out
    if (i != 10000)
    {
      //Accelerometer readings are 10-bit values sent as unsigned bytes.
      //First three bytes are 8 highest order bits
      uint8_t x_up = Wire.read();
      uint8_t y_up = Wire.read();
      uint8_t z_up = Wire.read();
      
      //Last three bytes are 2 lowest order bits
      uint8_t x_low = Wire.read();
      uint8_t y_low = Wire.read();
      uint8_t z_low = Wire.read();
  
      //Combine high and low bytes (uses coercion to int)
      x = x_up > 127 ? ((x_up&0xFF) - 256) * 4 + x_low: x_up * 4 + x_low;
      if(x < -200) x = -200;
      else if (x > 200) x = 200;
    }
  }
  
  return x;
}

